#!/bin/sh
# the next line restarts using tclsh \
exec tclsh8.0 "$0" ${1+"$@"}

# twohtmlc.tcl --
#
#	"Compile" html files (and images): create tcl code which adds
#	the files (given on the command line) to an html widget
#
# Copyright (c) 1999 Frank J. Leitner
#
# See the file "license.terms" for information on usage and redistribution
# of this file, and for a DISCLAIMER OF ALL WARRANTIES.
#
# $Header: /home/ldv/cvs/two/bin/twohtmlc.tcl,v 1.5 1999/10/15 20:08:55 fjl Exp $

#-------------------------------------------------------------------------
# process options
#-------------------------------------------------------------------------

array set opt {
    verbose  0
    widget   ".html"
    type     widget
}

for {set idx 0} {$idx < [llength $argv]} {} {
    switch -exact -- [lindex $argv $idx] {
        -- {
	    set argv [lreplace $argv $idx $idx]
	    break
	}
        -? -
        -h -
	-he -
	-hel -
	-help {
	    puts {
Usage: twohtmlc.tcl [-help] [-verbose] [-array name] [-widget path] [-out outfile] infiles
  -h|-help:    Print this message and exit
  -v|-verbose: Be more verbose
  -a|-arrray:  Name of array which will hold all data
  -w|-widget:  Name of html widget which is to be configured
  -o|-out:     Select the file to print into
  The -a and -w options override each other.
};
	    exit
	}
	-v -
	-ve -
	-ver -
	-verb -
	-verbo -
	-verbos -
	-verbose {
	    set opt(verbose) 1
	    set argv [lreplace $argv $idx $idx]
	}
	-a -
	-ar -
	-arr -
	-arra -
	-array {
	    set argv [lreplace $argv $idx $idx]
	    set opt(array) [lindex $argv $idx]
	    set argv [lreplace $argv $idx $idx]
	    set opt(type) array
	}
	-w -
	-wi -
	-wid -
	-widg -
	-widge -
	-widget {
	    set argv [lreplace $argv $idx $idx]
	    set opt(widget) [lindex $argv $idx]
	    set argv [lreplace $argv $idx $idx]
	    set opt(type) widget
	}
	-o -
	-ou -
	-out {
	    set argv [lreplace $argv $idx $idx]
	    set opt(out) [lindex $argv $idx]
	    set argv [lreplace $argv $idx $idx]
	}
	default {
	    incr idx
	}
    }
}

# base64_encode --
#
# 	do base-64 encoding (see RFC 2045 for details). This version
#	works on strings; it transform the input string given as
#	argument into the base64-encoded version which is returned.
#	CAVEAT EMPTOR: When reading the input data from whatever
#	source (like a file), make sure that you don't use any kind
#	of EOL translation => fconfigure $fh -translation binary
#
# Arguments:
# 	orig -		binary data which is to be encoded
#
# Results:
#	base64-encoded string

proc base64_encode {orig} {

    set base64en {
        A B C D E F G H I J K L M N O P Q R S T U V W X Y Z
	a b c d e f g h i j k l m n o p q r s t u v w x y z
	0 1 2 3 4 5 6 7 8 9 + /
    }
    set pos 0
    set out ""

    foreach {i0 i1 i2} [split $orig {}] {
        set in "$i0$i1$i2"
        set num [string length $in]
	binary scan \000$in\000\000 I b32
	for {set idx 18} {$idx >= (3-$num)*6} {incr idx -6} {
	    append out [lindex $base64en [expr {($b32>>$idx) & 0x3F}]]
	}
	if {$num < 3} {
	    for {set idx 0} {$idx < 3-$num} {incr idx} {
	        append out "="
	    }
	    append out "\n"
	    set pos 0
	} elseif {$pos == 17} {
	    append out "\n"
	    set pos 0
	} else {
	    incr pos
	}
    }
    return $out
}

# where to output
if {[info exists opt(out)]} {
    set outfh [open $opt(out) w]
} else {
    set outfh stdout
}

# read files
foreach f $argv {
    if {$opt(verbose)} {
        puts $f
    }
    if {[string match "widget" $opt(type)]} {
        # widget name added only at the very end: we can't
        # insert in into the list as it might well contain
        # a variable (like $helpWidget) and the list command
        # would surround it with {}....
        set cmd [list add $f]
    } else {
        set cmd [list set $opt(array)($f)]
    }
    if {![file readable $f]} {
        puts stderr "Couldn't read \"$f\""
    } elseif {[file isdirectory $f]} {
        puts stderr "skipping directory \"$f\""
    } elseif {![file isfile $f]} {
        puts stderr "\"$f\" is no regular file"
    } elseif {[catch {set infh [open $f r]} err]} {
        puts stderr $err
    } else {
        switch -exact -- [string tolower [file extension $f]] {
            .html -
            .htm -
            .txt -
            .xpm -
            .xbm {
                set plain 1
            }
            .gif -
            .jpg -
            .jpeg {
                set plain 0
            }
            default {
                set plain 1
            }
        }
        if {$plain} {
            if {$opt(verbose)} {
                puts "-> plain"
            }
            lappend cmd [read $infh]
        } else {
            if {$opt(verbose)} {
                puts "-> binary"
            }
            fconfigure $infh -translation binary
            lappend cmd [base64_encode [read $infh]]
        }
        close $infh
        if {[string match "widget" $opt(type)]} {
            puts $outfh "$opt(widget) $cmd"
        } else {
            puts $outfh $cmd
        }
    }
}

# close output stream
if {[info exists opt(out)]} {
    close $outfh
}
