#!/bin/sh
# the next line restarts using tclsh \
exec tclsh8.0 "$0" ${1+"$@"}

# twopre.tcl --
#
#	"Precompile" tcl file:
#	 - inlining of "source" (with nesting)
#	 - inlining of "image create ... -file" as "image create ... -data"
#	 - optionally remove comments
#	 - process "two_{define|undef|ifdef|ifndef|else/endif}"
#
# Copyright (c) 1998 Frank J. Leitner
#
# See the file "license.terms" for information on usage and redistribution
# of this file, and for a DISCLAIMER OF ALL WARRANTIES.
#
# $Header: /home/ldv/cvs/two/bin/twopre.tcl,v 1.6 1999/12/09 15:18:01 fjl Exp $

#-------------------------------------------------------------------------
# process options
#-------------------------------------------------------------------------

set opt(verbose) 0
set opt(strip) 0
array set TwoPre ""
array set TwoPreFixed ""

for {set idx 0} {$idx < [llength $argv]} {} {
    switch -exact -- [lindex $argv $idx] {
        -- {
	    set argv [lreplace $argv $idx $idx]
	    break
	}
        -? -
        -h -
	-he -
	-hel -
	-help {
	    puts "\
Usage: twopre.tcl [-help] [-verbose] [-strip]
                  [-DEFINE macro] [-UNDEF macro]
		  [infile [outfile]]
  -h|-help:    Print this message and exit
  -v|-verbose: Be more verbose
  -s|-strip:   Remove comments
  -D|-DEFINE:  Define macro
  -U|-UNDEF:   Undefine macro
";
	    exit
	}
	-v -
	-ve -
	-ver -
	-verb -
	-verbo -
	-verbos -
	-verbose {
	    set opt(verbose) 1
	    set argv [lreplace $argv $idx $idx]
	}
	-s -
	-st -
	-str -
	-stri - 
	-strip {
	    set opt(strip) 1
	    set argv [lreplace $argv $idx $idx]
	}
	-D -
	-DE -
	-DEF -
	-DEFI -
	-DEFIN -
	-DEFINE {
	    set next [expr {$idx + 1}]
	    set val [lindex $argv $next]
	    set TwoPre($val) 1
	    set TwoPreFixed($val) ""
	    set argv [lreplace $argv $idx $next]
	}
	-U -
	-UN -
	-UND -
	-UNDE -
	-UNDEF {
	    set next [expr {$idx + 1}]
	    set val [lindex $argv $next]
	    set TwoPre($val) 0
	    set TwoPreFixed($val) ""
	    set argv [lreplace $argv $idx $next]
	}
	default {
	    incr idx
	}
    }
}

#-------------------------------------------------------------------------
# helper functions
#-------------------------------------------------------------------------

# parse --
#
#	Parse the string and try to recognise all two_pre stuff
#
# Argumets:
#	level -		Level of indendation

proc parse {in} {
    global opt TwoPre TwpPreFixed

    # just for performance's sake
    if {![string length $in]} {
        return ""
    }
    set ws "\[ \t\]"

    # no more need for package require TwoPre
    regsub -all "package$ws+require$ws+TwoPre($ws+\[0-9.\]+)?" $in {} in

    set tokenRE {two_define|two_undef|two_ifdef|two_ifndef|source|image}
    set startRE "^|\n|;"

    set old  $in
    set result  ""

    # search for next potential token
    while {[regexp "($startRE)($ws*)($tokenRE)($ws.*)\$" $in in pre1 pre2 token tail]} {
        # append anything before this token to the result
	append result [string range $old 0 \
	    [expr {[string length $old]-[string length $in]-1}] ] $pre1 $pre2

	set cmd [getFirstCmd "$token$tail"]
	# in for next round is only the remaining part
	set in [string range $tail \
	    [expr {[string length $cmd]-[string length $token]}] end]
        # all command have at least one option...
	set opt1 [lindex $cmd 1]
	switch [lindex $cmd 0] {
	    two_define {
		if {$opt(verbose)} {
	            puts [lrange $cmd 0 1]
		}
		if {![info exists TwoPreFixed($opt1)]} {
		    set TwoPre($opt1) 1
		}
	    }
	    two_undef {
		if {$opt(verbose)} {
	            puts [lrange $cmd 0 1]
		}
		if {![info exists TwoPreFixed($opt1)]} {
		    set TwoPre($opt1) 0
		}
	    }
	    two_ifdef {
		if {$opt(verbose)} {
	            puts [lrange $cmd 0 1]
		}
		if {[info exists TwoPre($opt1)] && $TwoPre($opt1)} {
		    append result [parse [lindex $cmd 2]]
		} else {
		    if {[string match "two_else"  [lindex $cmd 3]] ||
			[string match "two_endif" [lindex $cmd 3]]
                    } {
			append result [parse [lindex $cmd 4]]
		    } else {
			append result [parse [lindex $cmd 3]]
		    }
		}
	    }
	    two_ifndef {
		if {$opt(verbose)} {
	            puts [lrange $cmd 0 1]
		}
		if {[info exists TwoPre($opt1)] && $TwoPre($opt1)} {
		    if {[string match "two_else"  [lindex $cmd 3]] ||
			[string match "two_endif" [lindex $cmd 3]]
		    } {
			append result [parse [lindex $cmd 4]]
		    } else {
			append result [parse [lindex $cmd 3]]
		    }
		} else {
		    append result [parse [lindex $cmd 2]]
		}
	    }
	    source {
		if {![string match pre.tcl [file tail $opt1]]} {
		    if {$opt(verbose)} {
	        	puts [lrange $cmd 0 1]
		    }
		    if {[catch {set fh [open $opt1]} msg]} {
			puts stderr $msg
		    } else {
			set data [read $fh]
			close $fh
			append result [parse $data]
		    }
		}
	    }
	    image {
		if {[string match create $opt1]} {
		    set idx [lsearch -exact $cmd "-file"]
		    if {$idx >= 0} {
		        set next [expr {$idx + 1}]
			if {$opt(verbose)} {
	        	    puts "[lindex $cmd 0] [lrange $cmd $idx $next]"
			}
			if {[catch {set fh [open [lindex $cmd $next]]} msg]} {
			    puts stderr $msg
			} else {
			    fconfigure $fh -translation binary
			    set data [read $fh]
			    close $fh
			    if {[string match photo [lindex $cmd 2]]} {
				set data [base64_encode $data]
			    } else {
				regsub -all "\#define" $data {\\\0} data
			    }
			    set cmd [lreplace $cmd $idx $next "-data" "\n$data"]
			}
		    }
		}
  	        append result $cmd
	    }
	}

        set old $in
    }
    append result $in
    return $result
}

# getFirstCmd --
#
# 	Get the first full command from the given string.
#
# Arguments:
# 	in -		input string
#
# Results:
#	first command as string

proc getFirstCmd {in} {
    set result ""

    set firstL 1
    foreach l [split $in "\n"] {
        if {$firstL} {
	    set firstL 0
	} else {
	    append result "\n"
	}
        set firstP 1
        foreach p [split $l ";"] {
	    if {$firstP} {
		set firstP 0
	    } else {
	        append result ";"
	    }
            append result $p
	    if {[info complete $result]} {
		return $result
	    }
	}
    }
    return $result
}


# base64_encode_fh --
#
# 	do base-64 encoding (see RFC 2045 for details). This version
#	works on two filehandles for inputr and output, so it is
#	suitable for larger files without taking lots of memory.
#
# Arguments:
# 	infh -		input filehandle
#	outfh -		output filehandle; defaults to stdout
#
# Sideeffects:
#	write to stdout
#
# Results:
#	none

proc base64_encode_fh {infh {outfh stdout}} {

    set base64en {
        A B C D E F G H I J K L M N O P Q R S T U V W X Y Z
	a b c d e f g h i j k l m n o p q r s t u v w x y z
	0 1 2 3 4 5 6 7 8 9 + /
    }
    set pos 0

    # make sure that no EOL translation takes place!
    fconfigure $infh -translation binary

    while {1} {
        set in [read $infh 3]
	set num [string length $in]
	binary scan \000$in\000\000 I b32
	for {set idx 18} {$idx >= (3-$num)*6} {incr idx -6} {
	    puts -nonewline $outfh [lindex $base64en [expr {($b32>>$idx) & 0x3F}]]
	}
	if {$num < 3} {
	    for {set idx 0} {$idx < 3-$num} {incr idx} {
	        puts -nonewline $outfh =
	    }
	    puts $outfh ""
	    set pos 0
	} elseif {$pos == 17} {
	    puts $outfh ""
	    set pos 0
	} else {
	    incr pos
	}
    }
    puts $outfh ""
    return
}

# base64_encode --
#
# 	do base-64 encoding (see RFC 2045 for details). This version
#	works on strings; it transform the input string given as
#	argument into the base64-encoded version which is returned.
#	CAVEAT EMPTOR: When reading the input data from whatever
#	source (like a file), make sure that you don't use any kind
#	of EOL translation => fconfigure $fh -translation binary
#
# Arguments:
# 	orig -		binary data which is to be encoded
#
# Results:
#	base64-encoded string

proc base64_encode {orig} {

    set base64en {
        A B C D E F G H I J K L M N O P Q R S T U V W X Y Z
	a b c d e f g h i j k l m n o p q r s t u v w x y z
	0 1 2 3 4 5 6 7 8 9 + /
    }
    set pos 0
    set out ""

    foreach {i0 i1 i2} [split $orig {}] {
        set in "$i0$i1$i2"
        set num [string length $in]
	binary scan \000$in\000\000 I b32
	for {set idx 18} {$idx >= (3-$num)*6} {incr idx -6} {
	    append out [lindex $base64en [expr {($b32>>$idx) & 0x3F}]]
	}
	if {$num < 3} {
	    for {set idx 0} {$idx < 3-$num} {incr idx} {
	        append out "="
	    }
	    append out "\n"
	    set pos 0
	} elseif {$pos == 17} {
	    append out "\n"
	    set pos 0
	} else {
	    incr pos
	}
    }
    return $out
}

#-------------------------------------------------------------------------
# init parsing
#-------------------------------------------------------------------------

# read input data
if {[llength $argv] >= 1} {
    set infh [open [lindex $argv 0] r]
    set in [read $infh]
    close $infh
} else {
    set in [read stdin]
}

# parse that thing
set out [parse $in]

if {$opt(strip)} {
    # strip comments and empty lines.
    # CAVEAT EMPTOR: This does NOT parse the language syntax; it
    # is merely a bunch of heuristics about where comments usually
    # appear! And empty lines could have a purpose as well (e.g. in
    # strings)
    # Comments starting with two "#" signs are not removed.
    while {[
        regsub -all "(^|\n|;)\[ \t\]*#(\[^!#\n\]\[^\r\n\]*)?(\n|\$)" $out {\1\3} out
    ]} {
    }
    set out2 ""
    foreach line [split $out "\n"] {
        if {[string length [string trimright $line]]} {
	    append out2 $line "\n"
	}
    }
    set out $out2
}

# write results
if {[llength $argv] >= 2} {
    set outfh [open [lindex $argv 1] w]
    puts -nonewline $outfh $out
    close $outfh
} else {
    puts -nonewline $out
}
