#! /usr/local/bin/wish

source ../lib/util.tcl
source ../lib/framework.tcl
source ../lib/splitter.tcl
source ../lib/text.tcl
source ../lib/autosb.tcl
source ../lib/editor.tcl

# Menus
menu .m
. configure -menu .m

menu .m.file -tearoff false -postcommand fileState
.m add cascade -menu .m.file -label "File"
.m.file add command -label "New" -command {.ed open}
.m.file add command -label "Open..." -command {set f [tk_getOpenFile]; if {$f != {}} {.ed open $f}}
.m.file add separator
.m.file add command -label "Save" -command {.ed save}
.m.file add command -label "Save as..." -command {set f [tk_getSaveFile]; if {$f != {}} {.ed save $f}}
.m.file add command -label "Close" -command {.ed close}
.m.file add separator
.m.file add command -label "Exit" -command exit

proc fileState {} {
    if [.ed modState] {
        .m.file entryconfigure 3 -state normal
    } else {
        .m.file entryconfigure 3 -state disabled
    }
}

menu .m.win -tearoff false -postcommand winState
.m add cascade -menu .m.win -label "Window"
.m.win add command -label "Split" -command {.ed configure -split 1}
.m.win add command -label "Join"  -command {.ed configure -split 0}

proc winState {} {
    if [.ed splitState] {
        .m.win entryconfigure 0 -state disabled
        .m.win entryconfigure 1 -state normal
    } else {
        .m.win entryconfigure 0 -state normal
        .m.win entryconfigure 1 -state disabled
    }
}

pack [frame .b] -side bottom -fill x
pack [button .b.c -text "Copy" -command {puts [.ed get sel.first sel.last]} -state disabled] -side left
pack [button .b.u -text "Undo" -command {.ed undo} -state disabled] -side left 
pack [button .b.r -text "Redo" -command {.ed redo} -state disabled] -side left
pack [label  .b.l -textvariable pos] -side right
pack [two_editor .ed] -fill both -expand 1 -side top

bind .ed <<SelOn>>   { .b.c configure -state normal   }
bind .ed <<SelOff>>  { .b.c configure -state disabled }
bind .ed <<UndoOn>>  { .b.u configure -state normal   }
bind .ed <<UndoOff>> { .b.u configure -state disabled }
bind .ed <<RedoOn>>  { .b.r configure -state normal   }
bind .ed <<RedoOff>> { .b.r configure -state disabled }
bind .ed <<PosChanged>> { set pos "X: %x Y: %y"}

set pos "X: 1 Y: 1"
focus .ed

.ed configure -highlight 1 -string {\"} -comment {\# \$} -keywords "abc"

if {[llength $argv]} {
    .ed open [lindex $argv 0]
}
