.TH two_mdi n multiple\ document\ interface\ widget TWO\ library
.ds )H FJL
.ds ]W http://fjl.de/two/
.SH NAME
two_mdi \- Multiple document interface widget
.SH SYNOPSIS
.CI package\ require\ twoMdi \ ?1.0?
.br
.C two_mdi
.I pathName
.RI ? options ?
.SH DESCRIPTION
.C two_mdi
is an implementation of the MDI widget known from Windows. An MDI is like a
windowmanager in another window: you can open an arbitrary number of
slave windows, move, resize, maximize and minimize them, and sure enough
close them. There are window menus and a root menu.
.PP
Creating a new
.C two_mdi
gives you a master widget which serves as a container for slave widgets.
Handling of slave windows is a bit weird with MDI. It doesn\'t work like
a TK geometry manager, and this the slave widgets are no direct children
of the master widget. There is a mdi command to create new slaves, which
will return the path to the frame.
.PP
MIDs are not really the UNIX (Motif?) way of handling more than one
document with one instance of an application. The most obvious disadvantage
is the loss of freedom (you can\'t place slaves independant from the
master). Yet there might be reasons to use this: you might want to run the
Tcl/Tk application on Windows, or the average user might be used to MDIs from
their windows applications and thus accept them even on UNIX, or you just
like them fore any reason.
.SH COMMANDS
The
.C two_mdi
widget command takes the following commands (as the first argument):
.RS
.TP
.IC pathName\  addSlave \ ?slaveoptions?
Add a slave widget. Returns path of the new slave widget (a frame widget).
Destroying this widget will clean up things, but won\'t call the
closecmd. Better use 
.C master\ delete
.CR slave .
.TP
.IC pathName\  new \ ?slaveoptions?
Just an alias for
.CR addSlave .
.TP
.IC pathName\  slaves
Returns a list of all slave widgets\' pathnames.
.TP
.IC pathName\  entryconfigure \ slavePathName\ ?options?
Configures
.I options
for the slave widget 
.IR slavePathName .
.TP
.IC pathName\  entrycget \ slavePathName\ ?option?
Returns configuration setting of
.C option
from slave widget
.IR slavePathName .
.TP
.IC pathName\  active
Returns path to currently active child (or an empty string).
.TP
.IC pathName\  tile
Tile the slave widgets (fill all space of the master with all slaves).
.TP
.IC pathName\  cascade
Cascade the slave widgets (stack them with some displacement).
.TP
.IC pathName\  show \ slavePathName
Show slave widget 
.IR slavePathName ,
i.e. raise it, maybe restore from minimized state, and set the focus.
.TP
.IC pathName\  delete \ slavePathName
Delete slave widget denoted by
.IR slavePathName.
.TP
.IC pathName\  maximize
Maximize all slave widgets.
.TP
.IC pathName\  minimize \ ?slavePathName...?
Minimize all given slave widgets
.IR slavePathName .
If no slaves are given: minimize all.
.TP
.IC pathName\  restore \ ?slavePathName...?
Restore the original size of all maximized or minimized
widgets from the given list of slave widgets
.IR slavePathName .
If no slaves are given: restore all.
.RE
.SS SLAVE OPTIONS
You can apply the following options to the
.CR addSlave ,
.C entryconfigure
and
.C entrycget
commands:
.RS
.TP
.CI -title \ string
Set title of slave widget.
.TP
.CI -closecmd \ command
Invoke this command when slave widget is to be deleted. The actual
command will be
.C command
.IR slave\  ,
i.e. the name of the current
slave will be appended. The widget will only be destroyed if
this command returns TRUE (or no command is given).
.RE
.SH OPTIONS
The
.C two_mdi
command takes all options of the TK frame widget plus to following options:
.RS
.TP
.CI \-slavebg \ color
Background color of slave window.
.TP
.CI \-slavebd \ color
Border width of slave window\'s frame.
.TP
.CI \-font \ font
Font of title of slave window.
.TP
.CI \-titlebg \ color
Background color of an inactive slave\'s title.
.TP
.CI \-titlefg \ color
Foreground color of an inactive slave\'s title.
.TP
.CI \-activebg \ color
Background color of an active title.
.TP
.CI \-activefg \ color
Foreground color of an active title.
.TP
.CI \-xscrollcommand \ command
Specifies the
.I command
to be called on x-scroll. Should almost always be something like
.CR .xscrollbar\ set .
.TP
.CI \-yscrollcommand \ command
Specifies the
.I command
to be called on y-scroll. Should almost always be something like
.CR .yscrollbar\ set .
.TP
.CI \-arrangeicons \ TCL_BOOLEAN
If set to 1, then resizing the master and minimizing/restoring
the slaves will lead to a recalculation of all icon positions.
Defaults to 1.
.TP
.CI \-style \ style
Set the
.I style
of the MDI:
.C unix
for a more UNIX (KDE) like look or
.C windows
for a look like on windows.
.RE
.SH VIRTUAL EVENTS
The
.C two_mdi
master widget may generate the following virtual event:
.RS
.TP
.IR <<ActiveOn>>\  and\  <<ActiveOff>>
There is an active slave (On) oder not (Off). This event is
triggered as well whenever the focus is set to another slave.
.RE

The
.C TwoMdiSlave
slave widgets may generate the following virtual event:
.RS
.TP
.I <<Closeing>>
when they are destroyed.
.RE
.SH ELEMENTS
A
.C two_mdi
widget is built up with a rather complex widget hierarchy.
The master widget is a frame (class:
.IR TwoMdiMaster ).
Its children are the two popup menus (root
menu and window/icon menu).
.PP
The slave widgets\' hierarchy starts with a frame widget
serving as border (class:
.IR TwoMdi ).
This contains another frame widget (class again
.IR TwoMdi )
which provides the space (by padding) for further content.
This frame contains another three frames for
title (class:
.IR TwoMdiTitle ),
spaceing (class once more
.IR TwoMdi )
and the slave container returned to the user (class
.IR TwoMdiSlave ).
.PP
The title bar of the slaves is built with two labels
(the window icon and the title text) and three buttons
(minimize/restor, maximize/restore and close).
.SH AUTHOR
Frank J. Leitner
.br
frank@fjl.de
.SH SEE ALSO
frame(n).
