# bugmail.tcl --
#
#	bgerror replacement which enables the user to mail the stack
#	trace (and some more information) to the author of the
#	application (or whoever has been configured).
#
# Copyright (c) 1998 Frank J. Leitner
#
# See the file "license.terms" for information on usage and redistribution
# of this file, and for a DISCLAIMER OF ALL WARRANTIES.
#
# $Header: /home/ldv/cvs/two/lib/bugmail.tcl,v 1.23 1999/10/26 13:12:14 fjl Exp $

package provide twoBugmail 1.0
package require twoUtil

option add *TwoBugMail.message.txt.wrapLength   4i      widgetDefault
option add *TwoBugMail.message.bmp.bitmap       error   widgetDefault
option add *TwoBugMail.details.txt.relief       sunken  widgetDefault
option add *TwoBugMail.details.txt.borderWidth  1       widgetDefault
option add *TwoBugMail.details.txt.width        60      widgetDefault
option add *TwoBugMail.details.txt.height       20      widgetDefault
option add *TwoBugMail.txt.relief               sunken  widgetDefault
option add *TwoBugMail.txt.borderWidth          1       widgetDefault
option add *TwoBugMail.txt.width                40      widgetDefault
option add *TwoBugMail.txt.height               10      widgetDefault

namespace eval ::two::bugmail {
    namespace export two_bugmail
    variable Bugmail
    
    namespace import ::two::util::two_parseArgs
    
    # variable Bugmail
    #
    # from -	mail from
    # to -		mail to
    # cc -		mail cc
    # bcc -		mail bcc
    # subject -	mail subject
    # detail -	enable/disble viewing of error details
    # version -	additinal infos about the version of the software used
    # info -	more info about current version
    # installed -	true if bugmail is installed
    # button -	name of button pressed in dialog
    
    array set Bugmail {
        from		""
        to		""
        cc		""
        bcc		""
        subject		"[Error] %M"
        detail		1
        version		"unknown"
        installed	0
        button		""
        info		""
    }
    
    # variable BugmailMsg
    #
    
    array set BugmailMsg {
        errorTitle	"Internal error"
        errorIntro	"Internal error: "
        bugTitle	"Bug report"
        bugInfo	"You can mail a bug report to the author of this Software."
        mail	"Mail bug report"
        send	"Send mail"
        description	"Please enter a short description for the error. When and where did it happen?"
        details	"Details"
        ok		"Ok"
        cancel	"Cancel"
        ignore	"Ignore"
    }
}

# ::two::bugmail::two_bugmail --
#
#	create widget ._bugmail if it doesn't exist and parse argumets.
#
# Arguments:
#	opt		Method
#	args		Options

proc ::two::bugmail::two_bugmail {opt args} {
    variable Bugmail
    variable BugmailMsg

    switch -exact -- $opt {
        install {
	    two_parseArgs args Bugmail(from)    "-from"
	    two_parseArgs args Bugmail(to)      "-to"
	    two_parseArgs args Bugmail(cc)      "-cc"
	    two_parseArgs args Bugmail(bcc)     "-bcc"
	    two_parseArgs args Bugmail(subject) "-subject"
	    two_parseArgs args Bugmail(detail)  "-detail"
	    two_parseArgs args Bugmail(version) "-version"
	    two_parseArgs args Bugmail(info)    "-info"
	    if {[two_parseArgs args msg "-messages"]} {
	        array set BugmailMsg $msg
	    }
	    if [llength $args] {
	        error "Unknown arguments \"$args\""
	    }
	    if {!$Bugmail(installed)} {
		namespace eval :: "
        	    proc bgerror {error} {
			[namespace code ErrorHandler] \$error
		    }
		"
		set Bugmail(installed) 1
	    }
	}
	configure {
	    two_parseArgs args Bugmail(from)    "-from"
	    two_parseArgs args Bugmail(to)      "-to"
	    two_parseArgs args Bugmail(cc)      "-cc"
	    two_parseArgs args Bugmail(bcc)     "-bcc"
	    two_parseArgs args Bugmail(subject) "-subject"
	    two_parseArgs args Bugmail(detail)  "-detail"
	    two_parseArgs args Bugmail(version) "-version"
	    two_parseArgs args Bugmail(info)    "-info"
	    if {[two_parseArgs args msg "-messages"]} {
	        array set BugmailMsg $msg
	    }
	    if {[llength $args]} {
	        error "Unknown arguments \"$args\""
	    }
	}
	cget {
	    switch -exact -- $args {
	        -from     { return $Bugmail(from)    }
	        -to       { return $Bugmail(to)      }
	        -cc       { return $Bugmail(cc)      }
	        -bcc      { return $Bugmail(bcc)     }
	        -subject  { return $Bugmail(subject) }
	        -detail   { return $Bugmail(detail)  }
	    	-version  { return $Bugmail(version) }
	    	-info     { return $Bugmail(info)    }
	    	-messages { return [array get BugmailMsg]   }
		default   { error "Unkown option \"$args\"" }
	    }
	}
	remove {
	    if {$Bugmail(installed)} {
	        rename bgerror ""
		set Bugmail(installed) 0
	    }
	}
	comment {
	    ErrorHandler
	}
	default {
	    error "Unknown method \"$opt\""
	}
    }
    return
}

# ::two::bugmail::ErrorHandler --
#
#	takes error and shows description. O.k., it's a lengthy procedure:-)
#	Depending on configuration, this handler will allow the user to
#	send a bug report via e-mail. Currently, only UNIX using sendmail
#	is supported; directly implementing SMTP would need to configure
#	the smtp host - having to configure something in order to be able to
#	send a bug report isn't the best solution.
#	If no error is given, the user can still send a bug report.
#	See comments in code for single steps...
#
# Arguments:
#	err		Short error message

proc ::two::bugmail::ErrorHandler { {err ""} } {
    variable Bugmail
    variable BugmailMsg
    global errorInfo argv0 argv tcl_patchLevel tk_patchLevel

    # ------------------
    # Show error message
    # ------------------

    # bug text
    if {[string length $err]} {
	set title   $BugmailMsg(errorTitle)
	set text    $BugmailMsg(errorIntro)$err
	set details $errorInfo
	set errmsg  $err
	set ok      $BugmailMsg(ok)
    } else {
        set title   $BugmailMsg(bugTitle)
	set text    $BugmailMsg(bugInfo)
	set details "none"
	set errmsg  "Bug report"
	set ok      $BugmailMsg(cancel)
    }

    set w .twoBugmailWindow
    # remove old instances
    catch {destroy $w}
    # build widget
    toplevel     $w -class TwoBugMail
    wm minsize   $w 430 150
    wm title     $w $title
    wm iconname  $w $title
    wm transient $w .

    # 3 parts, details originally not visible    
    pack [frame $w.message -relief raised -bd 1] -side top -expand 1 -fill both
    frame $w.details -relief raised -bd 1
    pack [frame $w.buttons -relief raised -bd 1] -side bottom -fill both
    set msg $w.message
    set but $w.buttons
    set det $w.details
    
    # upper part: error message
    pack [label $msg.bmp] -side left -padx 3m -pady 3m
    pack [label $msg.txt -justify left -text $text] \
        -side right -expand 1 -fill both -padx 3m -pady 3m

    # middle part: details
    grid [text $det.txt -xs "$det.sx set" -ys "$det.sy set" -wrap none -takefocus 0] \
         -row 0 -column 0 -sticky nsew
    grid [scrollbar $det.sy -command "$det.txt yview"] \
         -row 0 -column 1 -sticky ns
    grid [scrollbar $det.sx -command "$det.txt xview" -orient horizontal] \
         -row 1 -column 0 -sticky ew
    grid rowconfigure    $det 0 -weight 1
    grid columnconfigure $det 0 -weight 1
    bindtags $det.txt [list $det.txt $w all]

    # fill in bug info (stack trace)
    $det.txt insert 1.0 $details
    $det.txt mark set insert 1.0

    # lower part: buttons
    button $but.ok -text $ok -underline 0 -default active -command \
        "[namespace code set] Bugmail(button) ok"
    button $but.ignore -text $BugmailMsg(ignore) -underline 0 -command \
        "[namespace code set] Bugmail(button) ignore"
    button $but.details -text $BugmailMsg(details) -underline 0 -command "
        pack $msg -fill x -expand 0
	pack forget $but.details
	pack $det -side bottom -expand 1 -fill both
    "
    button $but.mail -text $BugmailMsg(mail) -underline 0 -command \
        "[namespace code set] Bugmail(button) mail"
    if {$Bugmail(detail) && [string length $err]} {
	pack $but.ok $but.details $but.mail \
            -side left -padx 5 -fill x -expand 1
    } else {
	pack $but.ok $but.mail \
            -side left -padx 5 -fill x -expand 1
    }
    # ignore not packages: "return -code break" doesn't seem to work:-(

    # bindings for buttons
    foreach b {ok ignore details mail} {
        set key [string tolower [string index [$but.$b cget -text] 0]]
        bind $w <Alt-$key> "two_invoke $but.$b"
    }
    bind $w <Return>  "two_invoke $but.ok"
    bind $w <Escape>  "two_invoke $but.ignore"
    bind $w <Destroy> "[namespace code set] Bugmail(button) ignore"

    # Center the window on the screen.
    wm withdraw $w
    update idletasks
    set x [expr {[winfo screenwidth  $w]/2 - [winfo reqwidth  $w]/2 - [winfo vrootx $w]}]
    set y [expr {[winfo screenheight $w]/2 - [winfo reqheight $w]/2 - [winfo vrooty $w]}]
    wm geom $w +$x+$y
    wm deiconify $w

    # Grab and claim focus
    grab  $w
    focus $w
    
    # Wait for response
    tkwait variable [namespace current]::Bugmail(button)
    
    # User didn't want to send mail:
    if {[string match "ok" $Bugmail(button)]} {
	# Clean up
	catch {
            bind $w <Destroy> {}
            destroy $w
	}
        return
    } elseif {[string match "ignore" $Bugmail(button)]} {
	catch {
            bind $w <Destroy> {}
            destroy $w
	}
        return
    }

    # ---------------
    # Send bug report
    # ---------------

    # adopt window
    $msg.txt configure -text $BugmailMsg(description)
    pack $msg -fill x -expand 0
    $det.txt delete 1.0 end
    $det.txt configure -takefocus 1
    bindtags $det.txt [list $det.txt Text $w all]

    $but.ok     configure -text $BugmailMsg(send)
    $but.ignore configure -text $BugmailMsg(cancel)

    # clear former bindings
    foreach binding [bind $w] {
        bind $w $binding {}
    }

    # new bindings
    foreach b {ok ignore} {
        set key [string tolower [string index [$but.$b cget -text] 0]]
        bind $w <Alt-$key> "two_invoke $but.$b"
    }
    bind $det.txt <Escape> "two_invoke $but.ignore"
    focus $det.txt
    pack $det -side bottom -expand 1 -fill both
    pack forget $but.ok $but.ignore $but.details $but.mail
    pack $but.ok $but.ignore \
        -side left -padx 5 -fill x -expand 1

    # Wait for response
    tkwait variable [namespace current]::Bugmail(button)

    # get Users's description
    set description [$det.txt get 1.0 "end - 1c"]

    # Clean up
    catch {
        bind $w <Destroy> {}
        destroy $w
    }

    # User didn't want to send mail:
    if {[string match "ignore" $Bugmail(button)]} {
        return
    }

    # Send mail
    set fd [open "| /usr/lib/sendmail -oi -t" "w"]
    if {[string length $Bugmail(to)]} {
        puts $fd  "To: $Bugmail(to)"
    }
    if {[string length  $Bugmail(from)]} {
        puts $fd "From: $Bugmail(from)"
    }
    if {[string length $Bugmail(cc)]} {
        puts $fd  "Cc: $Bugmail(cc)"
    }
    if {[string length $Bugmail(bcc)]} {
        puts $fd  "Cc: $Bugmail(bcc)"
    }
    set subject "Error: $errmsg"
    if {[string length   $Bugmail(subject)]} {
	regsub -all {%M} $Bugmail(subject) $errmsg subject
    }
    puts $fd \
"Subject: $subject
Date: [clock format [clock seconds]]

The following error appeared:
-----------------------------------------------------------------
$errmsg
-----------------------------------------------------------------

Where and when:
-----------------------------------------------------------------
Prog: $argv0 $argv
Vers: $Bugmail(version)
Time: [clock format [clock seconds]]
Host: [info host]
PWD:  [pwd]
Tcl:  $tcl_patchLevel
Tk:   $tk_patchLevel
-----------------------------------------------------------------

Author-supplied info:
-----------------------------------------------------------------
$Bugmail(info)
-----------------------------------------------------------------

User-supplied comment:
-----------------------------------------------------------------
$description
-----------------------------------------------------------------

Stack trace:
-----------------------------------------------------------------
$details
-----------------------------------------------------------------
"

    close $fd

    return
}

namespace import ::two::bugmail::*
